const fs = require('fs');
const path = require('path');

function extract(atlasFile, extensions = ['.png', '.jpg', '.jpeg']) {
    const atlasData = fs.readFileSync(atlasFile);
    const rawPagesData = [];

    let rawPageData = [];
    const pagesData = [];

    atlasData
        .toString()
        .split("\n")
        .forEach((data) => {
            if (data.trim() === '') {
                if (rawPageData.length > 0) {
                    rawPagesData.push(rawPageData);
                    rawPageData = [];
                }
            } else {
                rawPageData.push(data);
            }
        });

    rawPagesData.forEach((rawPageData) => {
        let textureExt = null;
        let isFrame = false;
    
        const atlasData = {
            frames: []
        };
    
        let frameData = {};
    
        rawPageData.forEach((line, index) => {
            line = line.trim();
    
            if (index === 0) {
                textureExt = path.extname(line);
    
                if (extensions.includes(textureExt)) {
                    atlasData['atlasName'] = line.trim();
                }
            } else {
                let [key, value] = line.split(':');
                key = key.trim();
    
                if (value === undefined) {
                    isFrame = true;
    
                    if (frameData.key) {
                        atlasData.frames.push(frameData);
                        frameData = {};
                    }
    
                    frameData['key'] = `${key}${textureExt}`;
    
                } else {
                    value = parseValue(value.trim());
                    
                    if (isFrame) {
                        frameData[key] = value;
                    } else {
                        atlasData[key] = value;
                    }
                }
            }
        });
    
        pagesData.push(atlasData);
    });

    return pagesData;
}

function parseValue(value) {
    const arrayData = value.split(',');

    if (arrayData.length > 1) {
        return arrayData.map((value) => parseValue(value.trim()));

    } 

    if (value === 'false') {
        return false;
    }
    
    if (value === 'true') {
        return true;
    }

    if (!isNaN(parseInt(value))) {
        if (parseInt(value).toString().length === value.length) {
            return parseInt(value);
        }
    }

    return value;
};

module.exports = {
    extract
};